@@ -16,20 +16,20 @@
  *
  */
 use std::collections::HashMap;
-use std::fs::File;
-use std::io::{BufRead, BufReader, IsTerminal};
+use std::fs;
 use std::path::{Path, PathBuf};
 use std::time::Duration;
-use std::{env, fs, io};
 
 use clap::ArgMatches;
+use hurl::pretty::PrettyMode;
 use hurl::runner::Value;
 use hurl_core::input::Input;
-use hurl_core::typing::{BytesPerSec, Count, DurationUnit};
+use hurl_core::types::{BytesPerSec, Count, DurationUnit};
 
-use crate::cli::options::{
-    duration, variables, CliOptionsError, ErrorFormat, HttpVersion, IpResolve, Output,
-};
+use super::context::RunContext;
+use super::variables::TypeKind;
+use super::variables_file::VariablesFile;
+use super::{duration, variables, CliOptionsError, ErrorFormat, HttpVersion, IpResolve, Output};
 use crate::cli::OutputType;
 
 pub fn cacert_file(arg_matches: &ArgMatches) -> Result<Option<String>, CliOptionsError> {
@@ -82,19 +82,16 @@ pub fn client_key_file(arg_matches: &ArgMatches) -> Result<Option<String>, CliOp
 }
 
 /// Returns true if Hurl output uses ANSI code and false otherwise.
-pub fn color(arg_matches: &ArgMatches) -> bool {
+///
+/// If it has no flags, we use the run `context` to determine if we use color or not.
+pub fn color(arg_matches: &ArgMatches, context: &RunContext) -> bool {
     if has_flag(arg_matches, "color") {
         return true;
     }
     if has_flag(arg_matches, "no_color") {
         return false;
     }
-    if let Ok(v) = env::var("NO_COLOR") {
-        if !v.is_empty() {
-            return false;
-        }
-    }
-    io::stdout().is_terminal()
+    context.is_with_color()
 }
 
 pub fn compressed(arg_matches: &ArgMatches) -> bool {
@@ -102,7 +99,7 @@ pub fn compressed(arg_matches: &ArgMatches) -> bool {
 }
 
 pub fn connect_timeout(arg_matches: &ArgMatches) -> Result<Duration, CliOptionsError> {
-    let s = get::<String>(arg_matches, "connect_timeout").unwrap_or_default();
+    let s = get::<String>(arg_matches, "connect_timeout").unwrap_or("300".to_string());
     get_duration(&s, DurationUnit::Second)
 }
 
@@ -127,15 +124,15 @@ pub fn curl_file(arg_matches: &ArgMatches) -> Option<PathBuf> {
 }
 
 pub fn delay(arg_matches: &ArgMatches) -> Result<Duration, CliOptionsError> {
-    let s = get::<String>(arg_matches, "delay").unwrap_or_default();
+    let s = get::<String>(arg_matches, "delay").unwrap_or("0".to_string());
     get_duration(&s, DurationUnit::MilliSecond)
 }
 
 pub fn error_format(arg_matches: &ArgMatches) -> ErrorFormat {
-    let error_format = get::<String>(arg_matches, "error_format");
-    match error_format.as_deref() {
-        Some("long") => ErrorFormat::Long,
-        Some("short") => ErrorFormat::Short,
+    let error_format = get::<String>(arg_matches, "error_format").unwrap_or("short".to_string());
+    match error_format.as_str() {
+        "long" => ErrorFormat::Long,
+        "short" => ErrorFormat::Short,
         _ => ErrorFormat::Short,
     }
 }
@@ -173,10 +170,10 @@ pub fn html_dir(arg_matches: &ArgMatches) -> Result<Option<PathBuf>, CliOptionsE
         } else if path.is_dir() {
             Ok(Some(path.to_path_buf()))
         } else {
-            return Err(CliOptionsError::Error(format!(
+            Err(CliOptionsError::Error(format!(
                 "{} is not a valid directory",
                 path.display()
-            )));
+            )))
         }
     } else {
         Ok(None)
@@ -207,14 +204,17 @@ pub fn include(arg_matches: &ArgMatches) -> bool {
 
 /// Returns true if we have at least one input files.
 /// The input file can be a file, the standard input, or a glob (even a glob returns empty results).
-pub fn has_input_files(arg_matches: &ArgMatches) -> bool {
+pub fn has_input_files(arg_matches: &ArgMatches, context: &RunContext) -> bool {
     get_strings(arg_matches, "input_files").is_some()
         || get_strings(arg_matches, "glob").is_some()
-        || !io::stdin().is_terminal()
+        || !context.is_stdin_term()
 }
 
 /// Returns the input files from the positional arguments and the glob options
-pub fn input_files(arg_matches: &ArgMatches) -> Result<Vec<Input>, CliOptionsError> {
+pub fn input_files(
+    arg_matches: &ArgMatches,
+    context: &RunContext,
+) -> Result<Vec<Input>, CliOptionsError> {
     let mut files = vec![];
     if let Some(filenames) = get_strings(arg_matches, "input_files") {
         for filename in &filenames {
@@ -233,7 +233,7 @@ pub fn input_files(arg_matches: &ArgMatches) -> Result<Vec<Input>, CliOptionsErr
     for filename in glob_files(arg_matches)? {
         files.push(filename);
     }
-    if files.is_empty() && !io::stdin().is_terminal() {
+    if files.is_empty() && !context.is_stdin_term() {
         let input = match Input::from_stdin() {
             Ok(input) => input,
             Err(err) => return Err(CliOptionsError::Error(err.to_string())),
@@ -293,7 +293,7 @@ pub fn max_filesize(arg_matches: &ArgMatches) -> Option<u64> {
 }
 
 pub fn max_redirect(arg_matches: &ArgMatches) -> Count {
-    match get::<i32>(arg_matches, "max_redirects").unwrap() {
+    match get::<i32>(arg_matches, "max_redirects").unwrap_or(50) {
         -1 => Count::Infinite,
         m => Count::Finite(m as usize),
     }
@@ -317,16 +317,20 @@ pub fn json_report_dir(arg_matches: &ArgMatches) -> Result<Option<PathBuf>, CliO
         } else if path.is_dir() {
             Ok(Some(path.to_path_buf()))
         } else {
-            return Err(CliOptionsError::Error(format!(
+            Err(CliOptionsError::Error(format!(
                 "{} is not a valid directory",
                 path.display()
-            )));
+            )))
         }
     } else {
         Ok(None)
     }
 }
 
+pub fn negotiate(arg_matches: &ArgMatches) -> bool {
+    has_flag(arg_matches, "negotiate")
+}
+
 pub fn netrc(arg_matches: &ArgMatches) -> bool {
     has_flag(arg_matches, "netrc")
 }
@@ -353,6 +357,10 @@ pub fn no_proxy(arg_matches: &ArgMatches) -> Option<String> {
     get::<String>(arg_matches, "noproxy")
 }
 
+pub fn ntlm(arg_matches: &ArgMatches) -> bool {
+    has_flag(arg_matches, "ntlm")
+}
+
 pub fn output(arg_matches: &ArgMatches) -> Option<Output> {
     get::<String>(arg_matches, "output").map(|filename| Output::new(&filename))
 }
@@ -375,8 +383,34 @@ pub fn path_as_is(arg_matches: &ArgMatches) -> bool {
     has_flag(arg_matches, "path_as_is")
 }
 
-pub fn progress_bar(arg_matches: &ArgMatches) -> bool {
-    test(arg_matches) && !interactive(arg_matches) && !is_ci() && io::stderr().is_terminal()
+pub fn pinned_pub_key(arg_matches: &ArgMatches) -> Option<String> {
+    get::<String>(arg_matches, "pinned_pub_key")
+}
+
+pub fn pretty(arg_matches: &ArgMatches, context: &RunContext) -> PrettyMode {
+    if has_flag(arg_matches, "pretty") {
+        return PrettyMode::Force;
+    }
+    if has_flag(arg_matches, "no_pretty") {
+        return PrettyMode::None;
+    }
+    if context.is_stdout_term() {
+        PrettyMode::Automatic
+    } else {
+        PrettyMode::None
+    }
+}
+
+pub fn progress_bar(arg_matches: &ArgMatches, context: &RunContext) -> bool {
+    // The test progress bar is displayed only for in test mode, for interactive TTYs.
+    // It can be forced by `--progress-bar` option.
+    if !test(arg_matches) {
+        return false;
+    }
+    if has_flag(arg_matches, "progress_bar") {
+        return true;
+    }
+    context.is_stderr_term() && !context.is_ci()
 }
 
 pub fn proxy(arg_matches: &ArgMatches) -> Option<String> {
@@ -404,30 +438,65 @@ pub fn retry(arg_matches: &ArgMatches) -> Option<Count> {
 }
 
 pub fn retry_interval(arg_matches: &ArgMatches) -> Result<Duration, CliOptionsError> {
-    let s = get::<String>(arg_matches, "retry_interval").unwrap_or_default();
+    let s = get::<String>(arg_matches, "retry_interval").unwrap_or("1000".to_string());
     get_duration(&s, DurationUnit::MilliSecond)
 }
 
-pub fn secret(matches: &ArgMatches) -> Result<HashMap<String, String>, CliOptionsError> {
-    let mut secrets = HashMap::new();
-    if let Some(secret) = get_strings(matches, "secret") {
-        for s in secret {
-            let inferred = false;
-            let (name, value) = variables::parse(&s, inferred)?;
-            // We check that there is no existing secrets
-            if secrets.contains_key(&name) {
-                return Err(CliOptionsError::Error(format!(
-                    "secret '{}' can't be reassigned",
-                    &name
-                )));
-            }
-            // Secrets can only be string.
-            if let Value::String(value) = value {
-                secrets.insert(name, value);
+pub fn secret(
+    matches: &ArgMatches,
+    context: &RunContext,
+) -> Result<HashMap<String, String>, CliOptionsError> {
+    let mut all_secrets = HashMap::new();
+
+    // Secrets are always parsed as string.
+    let type_kind = TypeKind::String;
+
+    // Insert environment secrets `HURL_SECRET_foo`
+    for (env_name, env_value) in context.secret_env_vars() {
+        let value = variables::parse_value(env_value, type_kind)?;
+        add_secret(&mut all_secrets, env_name.to_string(), value)?;
+    }
+
+    // Add secrets from files:
+    if let Some(filenames) = get_strings(matches, "secrets_file") {
+        for f in &filenames {
+            let filename = Path::new(f);
+            let vars = VariablesFile::open(filename, type_kind)?;
+            for var in vars {
+                let (name, value) = var?;
+                add_secret(&mut all_secrets, name, value)?;
             }
         }
     }
-    Ok(secrets)
+
+    // Finally, add single secrets.
+    if let Some(secrets) = get_strings(matches, "secret") {
+        for s in secrets {
+            let (name, value) = variables::parse(&s, type_kind)?;
+            add_secret(&mut all_secrets, name, value)?;
+        }
+    }
+    Ok(all_secrets)
+}
+
+/// Add a secret with `name` and `value` to the `secrets` hash map.
+fn add_secret(
+    secrets: &mut HashMap<String, String>,
+    name: String,
+    value: Value,
+) -> Result<(), CliOptionsError> {
+    // We check that there is no existing secrets
+    if secrets.contains_key(&name) {
+        return Err(CliOptionsError::Error(format!(
+            "secret '{}' can't be reassigned",
+            &name
+        )));
+    }
+    // Secrets can only be string.
+    if let Value::String(value) = value {
+        secrets.insert(name.to_string(), value);
+    }
+    Ok(())
 }
 
 pub fn ssl_no_revoke(arg_matches: &ArgMatches) -> bool {
@@ -443,7 +512,7 @@ pub fn test(arg_matches: &ArgMatches) -> bool {
 }
 
 pub fn timeout(arg_matches: &ArgMatches) -> Result<Duration, CliOptionsError> {
-    let s = get::<String>(arg_matches, "max_time").unwrap_or_default();
+    let s = get::<String>(arg_matches, "max_time").unwrap_or("300".to_string());
     get_duration(&s, DurationUnit::Second)
 }
 
@@ -464,56 +533,43 @@ pub fn user_agent(arg_matches: &ArgMatches) -> Option<String> {
 }
 
 /// Returns a map of variables from the command line options `matches`.
-pub fn variables(matches: &ArgMatches) -> Result<HashMap<String, Value>, CliOptionsError> {
+pub fn variables(
+    matches: &ArgMatches,
+    context: &RunContext,
+) -> Result<HashMap<String, Value>, CliOptionsError> {
     let mut variables = HashMap::new();
 
-    // Use environment variables prefix by HURL_
-    for (env_name, env_value) in env::vars() {
-        if let Some(name) = env_name.strip_prefix("HURL_") {
-            let inferred = true;
-            let value = variables::parse_value(env_value.as_str(), inferred)?;
-            variables.insert(name.to_string(), value);
-        }
+    // Variables are typed, based on their values.
+    let type_kind = TypeKind::Inferred;
+
+    // Insert environment variables `HURL_VARIABLE_foo`
+    for (env_name, env_value) in context.var_env_vars() {
+        let value = variables::parse_value(env_value, type_kind)?;
+        variables.insert(env_name.to_string(), value);
     }
 
-    if let Some(filenames) = get_strings(matches, "variables_file") {
-        for f in filenames.iter() {
-            let path = Path::new(&f);
-            if !path.exists() {
-                return Err(CliOptionsError::Error(format!(
-                    "Properties file {} does not exist",
-                    path.display()
-                )));
-            }
+    // Insert legacy environment variables `HURL_foo`
+    for (env_name, env_value) in context.legacy_var_env_vars() {
+        let value = variables::parse_value(env_value, type_kind)?;
+        variables.insert(env_name.to_string(), value);
+    }
 
-            let file = File::open(path).unwrap();
-            let reader = BufReader::new(file);
-            for (index, line) in reader.lines().enumerate() {
-                let line = match line {
-                    Ok(s) => s,
-                    Err(_) => {
-                        return Err(CliOptionsError::Error(format!(
-                            "Can not parse line {} of {}",
-                            index + 1,
-                            path.display()
-                        )))
-                    }
-                };
-                let line = line.trim();
-                if line.starts_with('#') || line.is_empty() {
-                    continue;
-                }
-                let inferred = true;
-                let (name, value) = variables::parse(line, inferred)?;
+    // Then add variables from files:
+    if let Some(filenames) = get_strings(matches, "variables_file") {
+        for f in &filenames {
+            let filename = Path::new(f);
+            let vars = VariablesFile::open(filename, type_kind)?;
+            for var in vars {
+                let (name, value) = var?;
                 variables.insert(name.to_string(), value);
             }
         }
     }
 
+    // Finally, add single variables from command line.
     if let Some(input) = get_strings(matches, "variable") {
         for s in input {
-            let inferred = true;
-            let (name, value) = variables::parse(&s, inferred)?;
+            let (name, value) = variables::parse(&s, type_kind)?;
             variables.insert(name.to_string(), value);
         }
     }
@@ -590,12 +646,7 @@ fn get_duration(s: &str, default_unit: DurationUnit) -> Result<Duration, CliOpti
         DurationUnit::MilliSecond => duration.value.as_u64(),
         DurationUnit::Second => duration.value.as_u64() * 1000,
         DurationUnit::Minute => duration.value.as_u64() * 1000 * 60,
+        DurationUnit::Hour => duration.value.as_u64() * 1000 * 60 * 60,
     };
     Ok(Duration::from_millis(millis))
 }
-
-/// Whether or not this running in a Continuous Integration environment.
-/// Code borrowed from <https://github.com/rust-lang/cargo/blob/master/crates/cargo-util/src/lib.rs>
-fn is_ci() -> bool {
-    env::var("CI").is_ok() || env::var("TF_BUILD").is_ok()
-}
