@@ -15,9 +15,13 @@
  * limitations under the License.
  *
  */
+use hurl_core::types::Index;
+
 /// This trait is implemented by run event observers, during the execution of one Hurl file.
 pub trait EventListener {
-    /// Call when running an entry, `entry_index` is the entry 0-based index in the Hurl file,
-    /// and `entry_count` is the total number of entries in the Hurl file.
-    fn on_running(&self, entry_index: usize, entry_count: usize);
+    /// Call when running an entry.
+    /// `current` is the entry index in the Hurl file,
+    /// `last` is the last entry index (may be less that the total number of entries).
+    /// `retry_count` is the current number of retries (i.e. 0 for a first run)
+    fn on_entry_running(&self, current: Index, last: Index, retry_count: usize);
 }
