@@ -20,18 +20,22 @@ use std::time::Instant;
 
 use chrono::Utc;
 use hurl_core::ast::{Entry, OptionKind, SourceInfo};
-use hurl_core::error::DisplaySourceError;
+use hurl_core::error::{DisplaySourceError, OutputFormat};
 use hurl_core::input::Input;
 use hurl_core::parser;
-use hurl_core::typing::Count;
+use hurl_core::types::{Count, Index};
 
 use crate::http::{Call, Client};
-use crate::runner::event::EventListener;
-use crate::runner::runner_options::RunnerOptions;
-use crate::runner::{entry, options, EntryResult, HurlResult, VariableSet};
 use crate::util::logger::{ErrorFormat, Logger, LoggerOptions};
 use crate::util::term::{Stderr, Stdout, WriteMode};
 
+use super::entry;
+use super::event::EventListener;
+use super::options;
+use super::result::{EntryResult, HurlResult};
+use super::runner_options::RunnerOptions;
+use super::variable::VariableSet;
+
 /// Runs a Hurl `content` and returns a [`HurlResult`] upon completion.
 ///
 /// If `content` is a syntactically correct Hurl file, an [`HurlResult`] is always returned on
@@ -39,6 +43,8 @@ use crate::util::term::{Stderr, Stdout, WriteMode};
 /// etc...) can be read in the [`HurlResult`] `success` field. If `content` is not syntactically
 /// correct, a parsing error is returned. This is the only possible way for this function to fail.
 ///
+/// The caller must ensure that `content` do not start with [BOM](https://en.wikipedia.org/wiki/Byte_order_mark).
+///
 /// `filename` indicates an optional file source, used when displaying errors.
 ///
 /// # Example
@@ -68,7 +74,7 @@ use crate::util::term::{Stderr, Stdout, WriteMode};
 ///
 /// // Set variables
 /// let mut variables = VariableSet::new();
-/// variables.insert("name".to_string(), Value::String("toto".to_string())).unwrap();
+/// variables.insert("name".to_string(), Value::String("toto".to_string()));
 ///
 /// // Run the Hurl sample
 /// let result = runner::run(
@@ -101,9 +107,16 @@ pub fn run(
     let hurl_file = parser::parse_hurl_file(content);
     let hurl_file = match hurl_file {
         Ok(h) => h,
-        Err(e) => {
-            logger.error_parsing_rich(content, filename, &e);
-            return Err(e.description());
+        Err(error) => {
+            let filename = filename.map_or(String::new(), |f| f.to_string());
+            let message = error.render(
+                &filename,
+                content,
+                None,
+                OutputFormat::Terminal(logger.color),
+            );
+            logger.error_rich(&message);
+            return Err(error.description());
         }
     };
 
@@ -144,12 +157,21 @@ pub fn run_entries(
     listener: Option<&dyn EventListener>,
     logger: &mut Logger,
 ) -> HurlResult {
+    // Early returns for empty entries.
+    if entries.is_empty() {
+        return HurlResult {
+            success: true,
+            variables: variables.clone(),
+            ..Default::default()
+        };
+    }
+
     let mut http_client = Client::new();
     let mut entries_result = vec![];
     let mut variables = variables.clone();
-    let mut entry_index = runner_options.from_entry.unwrap_or(1);
+    let mut current = Index::new(runner_options.from_entry.unwrap_or(1));
     let mut repeat_count = 0;
-    let n = runner_options.to_entry.unwrap_or(entries.len());
+    let last = Index::new(runner_options.to_entry.unwrap_or(entries.len()));
     let default_verbosity = logger.verbosity;
     let start = Instant::now();
     let timestamp = Utc::now().timestamp();
@@ -178,10 +200,10 @@ pub fn run_entries(
     // runner options and the "overridden" request options.
     // See <docs/spec/runner/run_cycle.md>
     loop {
-        if entry_index > n {
+        if current > last {
             break;
         }
-        let entry = &entries[entry_index - 1];
+        let entry = &entries[current.to_zero_based()];
 
         if let Some(pre_entry) = runner_options.pre_entry {
             let exit = pre_entry(entry);
@@ -199,11 +221,12 @@ pub fn run_entries(
             logger.verbosity = entry_verbosity;
         }
 
-        log_run_entry(entry_index, logger);
+        log_run_entry(current, logger);
 
-        // We can report the progression of the run for --test mode.
+        // We can report the progression of the run for --test mode. Fo this call, the retry count is
+        // zero, it will be potentially incremented in `run_request`.
         if let Some(listener) = listener {
-            listener.on_running(entry_index - 1, n);
+            listener.on_entry_running(current, last, 0);
         }
 
         // The real execution of the entry happens here, first: we compute the overridden request
@@ -213,7 +236,7 @@ pub fn run_entries(
             // If we have error evaluating request options, we consider it as a non retryable error
             // and either break the runner or go to the next entries.
             let entry_result = EntryResult {
-                entry_index,
+                entry_index: current,
                 source_info: entry.source_info(),
                 errors: vec![error.clone()],
                 ..Default::default()
@@ -221,7 +244,7 @@ pub fn run_entries(
             log_errors(&entry_result, content, filename, false, logger);
             entries_result.push(entry_result);
             if runner_options.continue_on_error {
-                entry_index += 1;
+                current += 1;
                 continue;
             } else {
                 break;
@@ -233,16 +256,16 @@ pub fn run_entries(
         // Should we skip?
         if options.skip {
             logger.debug("");
-            logger.debug_important(&format!("Entry {entry_index} has been skipped"));
-            entry_index += 1;
+            logger.debug_important(&format!("Entry {current} has been skipped"));
+            current += 1;
             continue;
         }
 
         // Repeat 0 is equivalent to skip.
         if options.repeat == Some(Count::Finite(0)) {
             logger.debug("");
-            logger.debug_important(&format!("Entry {entry_index} is skipped (repeat 0 times)"));
-            entry_index += 1;
+            logger.debug_important(&format!("Entry {current} is skipped (repeat 0 times)"));
+            current += 1;
             continue;
         }
 
@@ -251,7 +274,7 @@ pub fn run_entries(
         let delay_ms = delay.as_millis();
         if delay_ms > 0 {
             logger.debug("");
-            logger.debug_important(&format!("Delay entry {entry_index} (pause {delay_ms} ms)"));
+            logger.debug_important(&format!("Delay entry {current} (pause {delay_ms} ms)"));
             thread::sleep(delay);
         };
 
@@ -260,13 +283,15 @@ pub fn run_entries(
         // are not retried).
         let results = run_request(
             entry,
-            entry_index,
+            current,
+            last,
             content,
             filename,
             &mut http_client,
             &options,
             &mut variables,
             stdout,
+            listener,
             logger,
         );
 
@@ -289,26 +314,26 @@ pub fn run_entries(
         match options.repeat {
             None => {
                 repeat_count = 0;
-                entry_index += 1;
+                current += 1;
             }
             Some(Count::Finite(n)) => {
                 if repeat_count >= n {
                     repeat_count = 0;
-                    entry_index += 1;
+                    current += 1;
                 } else {
-                    logger.debug_important(&format!(
-                        "Repeat entry {entry_index} (x{repeat_count}/{n})"
-                    ));
+                    logger
+                        .debug_important(&format!("Repeat entry {current} (x{repeat_count}/{n})"));
                 }
             }
             Some(Count::Infinite) => {
-                logger.debug_important(&format!("Repeat entry {entry_index} (x{repeat_count})"));
+                logger.debug_important(&format!("Repeat entry {current} (x{repeat_count})"));
             }
         }
     }
 
     let duration = start.elapsed();
-    let cookies = http_client.cookie_storage(logger);
+    let cookie_store = http_client.cookie_store(logger);
+    let cookies = cookie_store.into_vec();
     let success = is_success(&entries_result);
     HurlResult {
         entries: entries_result,
@@ -322,30 +347,33 @@ pub fn run_entries(
 
 /// Runs an HTTP request and optional retry it until there are no HTTP errors. Returns a list of
 /// [`EntryResult`].
+/// `current` is the current index of the entry run, `last` is the index of the last entry to be run
 #[allow(clippy::too_many_arguments)]
 fn run_request(
     entry: &Entry,
-    entry_index: usize,
+    current: Index,
+    last: Index,
     content: &str,
     filename: Option<&Input>,
     http_client: &mut Client,
     options: &RunnerOptions,
     variables: &mut VariableSet,
     stdout: &mut Stdout,
+    listener: Option<&dyn EventListener>,
     logger: &mut Logger,
 ) -> Vec<EntryResult> {
     let mut results = vec![];
-    let mut retry_count = 1;
+    let mut retry_count = 0;
 
     loop {
-        let mut result = entry::run(entry, entry_index, http_client, variables, options, logger);
+        let mut result = entry::run(entry, current, http_client, variables, options, logger);
 
         // Check if we need to retry.
         let mut has_error = !result.errors.is_empty();
 
         // The retry threshold can only be reached with a finite positive number of retries
         let retry_max_reached = if let Some(Count::Finite(r)) = options.retry {
-            retry_count > r
+            retry_count >= r
         } else {
             false
         };
@@ -385,21 +413,29 @@ fn run_request(
         if !retry {
             break;
         }
-
+        retry_count += 1;
         let delay = options.retry_interval.as_millis();
+        let retry_max = match options.retry.unwrap() {
+            Count::Finite(max) => max.to_string(),
+            Count::Infinite => "ꝏ".to_string(),
+        };
         logger.debug("");
         logger.debug_important(&format!(
-            "Retry entry {entry_index} (x{retry_count} pause {delay} ms)"
+            "Retry on entry {current} (count: {retry_count}/{retry_max}, interval: {delay} ms)"
         ));
-        retry_count += 1;
-        // If we retry the entry, we do not want to display a 'blank' progress bar during the
-        // sleep delay. During the pause, we artificially show the previously erased progress
-        // line.
+
+        // If we're retrying, we report the running event just before sleeping in case the retry
+        // interval is too high: this way the visual retry hint will take the interval into account.
+        // (user will see "retry x" before going into sleep).
+        if let Some(listener) = listener {
+            listener.on_entry_running(current, last, retry_count);
+        }
+
         thread::sleep(options.retry_interval);
 
         // TODO: We keep this log because we don't want to change stderr with the changes
         // introduced by <https://github.com/Orange-OpenSource/hurl/issues/1973>
-        log_run_entry(entry_index, logger);
+        log_run_entry(current, logger);
     }
 
     results
@@ -547,16 +583,26 @@ fn log_errors(
 
     if logger.error_format == ErrorFormat::Long {
         if let Some(Call { response, .. }) = entry_result.calls.last() {
+            logger.info_curl_cmd(&entry_result.curl_cmd.to_string());
+            logger.info("");
+
             response.log_info_all(logger);
         }
     }
     entry_result.errors.iter().for_each(|error| {
-        logger.error_runtime_rich(content, filename, error, entry_result.source_info);
+        let filename = filename.map_or(String::new(), |f| f.to_string());
+        let message = error.render(
+            &filename,
+            content,
+            Some(entry_result.source_info),
+            OutputFormat::Terminal(logger.color),
+        );
+        logger.error_rich(&message);
     });
 }
 
 /// Logs the header indicating the begin of the entry run.
-fn log_run_entry(entry_index: usize, logger: &mut Logger) {
+fn log_run_entry(entry_index: Index, logger: &mut Logger) {
     logger.debug_important(
         "------------------------------------------------------------------------------",
     );
