@@ -15,6 +15,7 @@
  * limitations under the License.
  *
  */
+use chrono::Utc;
 use hurl_core::ast::{
     CertificateAttributeName, CookieAttribute, CookieAttributeName, CookiePath, Query, QueryValue,
     RegexValue, SourceInfo, Template,
@@ -22,63 +23,70 @@ use hurl_core::ast::{
 use regex::Regex;
 use sha2::Digest;
 
-use crate::http;
-use crate::runner::cache::BodyCache;
-use crate::runner::error::{RunnerError, RunnerErrorKind};
-use crate::runner::template::eval_template;
-use crate::runner::xpath::{Document, Format};
-use crate::runner::{filter, Number, Value, VariableSet};
+use crate::http::{HttpError, Response, ResponseCookie};
+
+use super::cache::BodyCache;
+use super::error::{RunnerError, RunnerErrorKind};
+use super::filter;
+use super::http_response::HttpResponse;
+use super::number::Number;
+use super::template::eval_template;
+use super::value::Value;
+use super::variable::VariableSet;
+use super::xpath::{Document, Format};
 
 pub type QueryResult = Result<Option<Value>, RunnerError>;
 
-/// Evaluates this `query` and returns a [`QueryResult`], using the HTTP `response` and `variables`.
+/// Evaluates this `query` and returns a [`QueryResult`], using a list of HTTP `responses` and `variables`.
 pub fn eval_query(
     query: &Query,
     variables: &VariableSet,
-    response: &http::Response,
+    responses: &[&Response],
     cache: &mut BodyCache,
 ) -> QueryResult {
+    let last_response = responses.last().unwrap();
     match &query.value {
-        QueryValue::Status => eval_query_status(response),
-        QueryValue::Version => eval_query_version(response),
-        QueryValue::Url => eval_query_url(response),
-        QueryValue::Header { name, .. } => eval_query_header(response, name, variables),
+        QueryValue::Status => eval_query_status(last_response),
+        QueryValue::Version => eval_query_version(last_response),
+        QueryValue::Url => eval_query_url(last_response),
+        QueryValue::Header { name, .. } => eval_query_header(last_response, name, variables),
         QueryValue::Cookie {
             expr: CookiePath { name, attribute },
             ..
-        } => eval_query_cookie(response, name, attribute, variables),
-        QueryValue::Body => eval_query_body(response, query.source_info),
+        } => eval_query_cookie(last_response, name, attribute, variables),
+        QueryValue::Body => eval_query_body(last_response, query.source_info),
         QueryValue::Xpath { expr, .. } => {
-            eval_query_xpath(response, cache, expr, variables, query.source_info)
+            eval_query_xpath(last_response, cache, expr, variables, query.source_info)
         }
         QueryValue::Jsonpath { expr, .. } => {
-            eval_query_jsonpath(response, cache, expr, variables, query.source_info)
+            eval_query_jsonpath(last_response, cache, expr, variables, query.source_info)
         }
         QueryValue::Regex { value, .. } => {
-            eval_query_regex(response, value, variables, query.source_info)
+            eval_query_regex(last_response, value, variables, query.source_info)
         }
         QueryValue::Variable { name, .. } => eval_query_variable(name, variables),
-        QueryValue::Duration => eval_query_duration(response),
-        QueryValue::Bytes => eval_query_bytes(response, query.source_info),
-        QueryValue::Sha256 => eval_query_sha256(response, query.source_info),
-        QueryValue::Md5 => eval_query_md5(response, query.source_info),
+        QueryValue::Duration => eval_query_duration(last_response),
+        QueryValue::Bytes => eval_query_bytes(last_response, query.source_info),
+        QueryValue::Sha256 => eval_query_sha256(last_response, query.source_info),
+        QueryValue::Md5 => eval_query_md5(last_response, query.source_info),
         QueryValue::Certificate {
             attribute_name: field,
             ..
-        } => eval_query_certificate(response, *field),
-        QueryValue::Ip => eval_ip(response),
+        } => eval_query_certificate(last_response, *field),
+        QueryValue::Ip => eval_ip(last_response),
+        QueryValue::Redirects => eval_redirects(responses),
     }
 }
 
 /// Evaluates the response status code using the HTTP `response`.
-fn eval_query_status(response: &http::Response) -> QueryResult {
+fn eval_query_status(response: &Response) -> QueryResult {
     Ok(Some(Value::Number(Number::Integer(i64::from(
         response.status,
     )))))
 }
 
 /// Evaluates the version on the HTTP `response`
-fn eval_query_version(response: &http::Response) -> QueryResult {
+fn eval_query_version(response: &Response) -> QueryResult {
     Ok(Some(Value::String(
         response
             .version
@@ -90,16 +98,12 @@ fn eval_query_version(response: &http::Response) -> QueryResult {
 }
 
 /// Evaluates the final URL of the HTTP `response`.
-fn eval_query_url(response: &http::Response) -> QueryResult {
+fn eval_query_url(response: &Response) -> QueryResult {
     Ok(Some(Value::String(response.url.to_string())))
 }
 
 /// Evaluates a response query header `name`, on the HTTP `response` given a set of `variables`.
-fn eval_query_header(
-    response: &http::Response,
-    name: &Template,
-    variables: &VariableSet,
-) -> QueryResult {
+fn eval_query_header(response: &Response, name: &Template, variables: &VariableSet) -> QueryResult {
     let name = eval_template(name, variables)?;
     let values = response.headers.values(&name);
     if values.is_empty() {
@@ -118,11 +122,12 @@ fn eval_query_header(
 
 /// Evaluates a cookie query `name` with optional attributes, on the HTTP `response` given a set of `variables`.
 fn eval_query_cookie(
-    response: &http::Response,
+    response: &Response,
     name: &Template,
     attribute: &Option<CookieAttribute>,
     variables: &VariableSet,
 ) -> QueryResult {
+    let query_source_info = name.source_info;
     let name = eval_template(name, variables)?;
     match response.get_cookie(&name) {
         None => Ok(None),
@@ -132,7 +137,7 @@ fn eval_query_cookie(
             } else {
                 CookieAttributeName::Value("Value".to_string())
             };
-            Ok(eval_cookie_attribute_name(attribute_name, cookie))
+            eval_cookie_attribute_name(attribute_name, cookie, query_source_info)
         }
     }
 }
@@ -140,7 +145,7 @@ fn eval_query_cookie(
 /// Evaluates the HTTP `response` body as text.
 ///
 /// `query_source_info` is the source position of the query, used if an error is returned.
-fn eval_query_body(response: &http::Response, query_source_info: SourceInfo) -> QueryResult {
+fn eval_query_body(response: &Response, query_source_info: SourceInfo) -> QueryResult {
     // Can return a string if encoding is known and utf8.
     match response.text() {
         Ok(s) => Ok(Some(Value::String(s))),
@@ -156,7 +161,7 @@ fn eval_query_body(response: &http::Response, query_source_info: SourceInfo) ->
 ///
 /// `query_source_info` is the source position of the query, used if an error is returned.
 fn eval_query_xpath(
-    response: &http::Response,
+    response: &Response,
     cache: &mut BodyCache,
     expr: &Template,
     variables: &VariableSet,
@@ -174,7 +179,7 @@ fn eval_query_xpath(
 ///
 /// `query_source_info` is used for error reporting.
 fn parse_cache_xml<'cache>(
-    response: &http::Response,
+    response: &Response,
     cache: &'cache mut BodyCache,
     query_source_info: SourceInfo,
 ) -> Result<&'cache Document, RunnerError> {
@@ -210,7 +215,7 @@ fn parse_cache_xml<'cache>(
 ///
 /// `query_source_info` is the source position of the query, used if an error is returned.
 fn eval_query_jsonpath(
-    response: &http::Response,
+    response: &Response,
     cache: &mut BodyCache,
     expr: &Template,
     variables: &VariableSet,
@@ -227,7 +232,7 @@ fn eval_query_jsonpath(
 ///
 /// `query_source_info` is used for error reporting.
 fn parse_cache_json<'cache>(
-    response: &http::Response,
+    response: &Response,
     cache: &'cache mut BodyCache,
     query_source_info: SourceInfo,
 ) -> Result<&'cache serde_json::Value, RunnerError> {
@@ -261,7 +266,7 @@ fn parse_cache_json<'cache>(
 ///
 /// `query_source_info` is the source position of the query, used if an error is returned.
 fn eval_query_regex(
-    response: &http::Response,
+    response: &Response,
     regex: &RegexValue,
     variables: &VariableSet,
     query_source_info: SourceInfo,
@@ -313,7 +318,7 @@ fn eval_query_variable(name: &Template, variables: &VariableSet) -> QueryResult
 
 /// Evaluates the effective duration of the HTTP `response` (only transfer time, assert and captures
 /// are not taken into account).
-fn eval_query_duration(response: &http::Response) -> QueryResult {
+fn eval_query_duration(response: &Response) -> QueryResult {
     Ok(Some(Value::Number(Number::Integer(
         response.duration.as_millis() as i64,
     ))))
@@ -322,7 +327,7 @@ fn eval_query_duration(response: &http::Response) -> QueryResult {
 /// Evaluates the HTTP `response` body as bytes.
 ///
 /// `query_source_info` is the source position of the query, used if an error is returned.
-fn eval_query_bytes(response: &http::Response, query_source_info: SourceInfo) -> QueryResult {
+fn eval_query_bytes(response: &Response, query_source_info: SourceInfo) -> QueryResult {
     match response.uncompress_body() {
         Ok(s) => Ok(Some(Value::Bytes(s))),
         Err(inner) => Err(RunnerError::new(
@@ -336,7 +341,7 @@ fn eval_query_bytes(response: &http::Response, query_source_info: SourceInfo) ->
 /// Evaluates the SHA-256 hash of the HTTP `response` body bytes.
 ///
 /// `query_source_info` is the source position of the query, used if an error is returned.
-fn eval_query_sha256(response: &http::Response, query_source_info: SourceInfo) -> QueryResult {
+fn eval_query_sha256(response: &Response, query_source_info: SourceInfo) -> QueryResult {
     let bytes = match response.uncompress_body() {
         Ok(s) => s,
         Err(inner) => {
@@ -357,7 +362,7 @@ fn eval_query_sha256(response: &http::Response, query_source_info: SourceInfo) -
 /// Evaluates the MD-5 hash of the HTTP `response` body bytes.
 ///
 /// `query_source_info` is the source position of the query, used if an error is returned.
-fn eval_query_md5(response: &http::Response, query_source_info: SourceInfo) -> QueryResult {
+fn eval_query_md5(response: &Response, query_source_info: SourceInfo) -> QueryResult {
     let bytes = match response.uncompress_body() {
         Ok(s) => s,
         Err(inner) => {
@@ -374,7 +379,7 @@ fn eval_query_md5(response: &http::Response, query_source_info: SourceInfo) -> Q
 
 /// Evaluates the SSL certificate attribute, of the HTTP `response`.
 fn eval_query_certificate(
-    response: &http::Response,
+    response: &Response,
     certificate_attribute: CertificateAttributeName,
 ) -> QueryResult {
     if let Some(certificate) = &response.certificate {
@@ -394,46 +399,78 @@ fn eval_query_certificate(
 }
 
 /// Evaluates the ip address of the HTTP `response`.
-fn eval_ip(response: &http::Response) -> QueryResult {
+fn eval_ip(response: &Response) -> QueryResult {
     Ok(Some(Value::String(response.ip_addr.to_string())))
 }
 
+/// Evaluates the redirects within a list of HTTP `responses`
+fn eval_redirects(responses: &[&Response]) -> QueryResult {
+    let mut it = responses.iter().peekable();
+    let mut values: Vec<Value> = vec![];
+    while let Some(r) = it.next() {
+        // We peek the next response in the response chain to use the next URL response
+        // as the current response location. The URLs' response chain has already been resolved
+        // we don't need to reconstruct the location from response headers.
+        let location = it.peek().map(|r| r.url.clone());
+        // We're only interested to redirection:
+        if location.is_some() {
+            let response = Value::HttpResponse(HttpResponse::new(location, r.status));
+            values.push(response);
+        }
+    }
+    Ok(Some(Value::List(values)))
+}
+
 fn eval_cookie_attribute_name(
     cookie_attribute_name: CookieAttributeName,
-    cookie: http::ResponseCookie,
-) -> Option<Value> {
+    cookie: ResponseCookie,
+    query_source_info: SourceInfo,
+) -> QueryResult {
     match cookie_attribute_name {
-        CookieAttributeName::Value(_) => Some(Value::String(cookie.value)),
+        CookieAttributeName::Value(_) => Ok(Some(Value::String(cookie.value))),
         CookieAttributeName::Expires(_) => {
             if let Some(s) = cookie.expires() {
-                match chrono::DateTime::parse_from_rfc2822(s.as_str()) {
-                    Ok(v) => Some(Value::Date(v.with_timezone(&chrono::Utc))),
-                    Err(_) => todo!(),
+                if let Ok(v) = chrono::DateTime::parse_from_rfc2822(s.as_str()) {
+                    Ok(Some(Value::Date(v.with_timezone(&chrono::Utc))))
+
+                // support format with dash such as Wed, 13-Jan-2021 22:23:01 GMT
+                // TODO: search for for other possible date format used in the wild
+                } else if let Ok(v) = chrono::NaiveDateTime::parse_from_str(
+                    s.as_str(),
+                    "%a, %d-%b-%Y %H:%M:%S%.3f GMT",
+                ) {
+                    Ok(Some(Value::Date(v.and_local_timezone(Utc).unwrap())))
+                } else {
+                    Err(RunnerError::new(
+                        query_source_info,
+                        RunnerErrorKind::Http(HttpError::CouldNotParseCookieExpires(s)),
+                        true,
+                    ))
                 }
             } else {
-                None
+                Ok(None)
             }
         }
         CookieAttributeName::MaxAge(_) => {
-            cookie.max_age().map(|v| Value::Number(Number::Integer(v)))
+            Ok(cookie.max_age().map(|v| Value::Number(Number::Integer(v))))
         }
-        CookieAttributeName::Domain(_) => cookie.domain().map(Value::String),
-        CookieAttributeName::Path(_) => cookie.path().map(Value::String),
+        CookieAttributeName::Domain(_) => Ok(cookie.domain().map(Value::String)),
+        CookieAttributeName::Path(_) => Ok(cookie.path().map(Value::String)),
         CookieAttributeName::Secure(_) => {
             if cookie.has_secure() {
-                Some(Value::Unit)
+                Ok(Some(Value::Unit))
             } else {
-                None
+                Ok(None)
             }
         }
         CookieAttributeName::HttpOnly(_) => {
             if cookie.has_httponly() {
-                Some(Value::Unit)
+                Ok(Some(Value::Unit))
             } else {
-                None
+                Ok(None)
             }
         }
-        CookieAttributeName::SameSite(_) => cookie.samesite().map(Value::String),
+        CookieAttributeName::SameSite(_) => Ok(cookie.samesite().map(Value::String)),
     }
 }
 
@@ -473,13 +510,14 @@ pub mod tests {
 
     use hurl_core::ast::{SourceInfo, TemplateElement, Whitespace};
     use hurl_core::reader::Pos;
-    use hurl_core::typing::ToSource;
+    use hurl_core::types::ToSource;
 
     use super::*;
+    use crate::http;
     use crate::http::{HeaderVec, HttpError, HttpVersion};
 
-    fn default_response() -> http::Response {
-        http::Response {
+    fn default_response() -> Response {
+        Response {
             version: HttpVersion::Http10,
             status: 200,
             headers: HeaderVec::new(),
@@ -700,7 +738,7 @@ pub mod tests {
                     value: QueryValue::Status,
                 },
                 &variables,
-                &http::hello_http_response(),
+                &[&http::hello_http_response()],
                 &mut cache,
             )
             .unwrap()
@@ -739,7 +777,7 @@ pub mod tests {
             eval_query(
                 &query_header,
                 &variables,
-                &http::hello_http_response(),
+                &[&http::hello_http_response()],
                 &mut cache
             )
             .unwrap(),
@@ -774,7 +812,7 @@ pub mod tests {
             eval_query(
                 &query_header,
                 &variables,
-                &http::hello_http_response(),
+                &[&http::hello_http_response()],
                 &mut cache
             )
             .unwrap()
@@ -795,11 +833,13 @@ pub mod tests {
         let mut headers = HeaderVec::new();
         headers.push(http::Header::new("Set-Cookie", "LSID=DQAAAKEaem_vYg; Path=/accounts; Expires=Wed, 13 Jan 2021 22:23:01 GMT; Secure; HttpOnly"));
 
-        let response = http::Response {
+        let response = Response {
             headers,
             ..default_response()
         };
 
+        let responses = vec![&response];
+
         // cookie "LSID"
         let query = Query {
             source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
@@ -819,7 +859,7 @@ pub mod tests {
             },
         };
         assert_eq!(
-            eval_query(&query, &variables, &response, &mut cache)
+            eval_query(&query, &variables, &responses, &mut cache)
                 .unwrap()
                 .unwrap(),
             Value::String("DQAAAKEaem_vYg".to_string())
@@ -848,7 +888,7 @@ pub mod tests {
             },
         };
         assert_eq!(
-            eval_query(&query, &variables, &response, &mut cache)
+            eval_query(&query, &variables, &responses, &mut cache)
                 .unwrap()
                 .unwrap(),
             Value::String("/accounts".to_string())
@@ -877,7 +917,7 @@ pub mod tests {
             },
         };
         assert_eq!(
-            eval_query(&query, &variables, &response, &mut cache)
+            eval_query(&query, &variables, &responses, &mut cache)
                 .unwrap()
                 .unwrap(),
             Value::Unit
@@ -906,14 +946,14 @@ pub mod tests {
             },
         };
         assert_eq!(
-            eval_query(&query, &variables, &response, &mut cache).unwrap(),
+            eval_query(&query, &variables, &responses, &mut cache).unwrap(),
             None
         );
     }
 
     #[test]
     fn test_eval_cookie_attribute_name() {
-        let cookie = http::ResponseCookie {
+        let cookie = ResponseCookie {
             name: "LSID".to_string(),
             value: "DQAAAKEaem_vYg".to_string(),
             attributes: vec![
@@ -935,35 +975,53 @@ pub mod tests {
                 },
             ],
         };
+        let query_source_info = SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0));
+
         assert_eq!(
-            eval_cookie_attribute_name(CookieAttributeName::Value("_".to_string()), cookie.clone())
-                .unwrap(),
+            eval_cookie_attribute_name(
+                CookieAttributeName::Value("_".to_string()),
+                cookie.clone(),
+                query_source_info
+            )
+            .unwrap()
+            .unwrap(),
             Value::String("DQAAAKEaem_vYg".to_string())
         );
         assert_eq!(
             eval_cookie_attribute_name(
                 CookieAttributeName::Domain("_".to_string()),
                 cookie.clone(),
-            ),
+                query_source_info
+            )
+            .unwrap(),
             None
         );
         assert_eq!(
-            eval_cookie_attribute_name(CookieAttributeName::Path("_".to_string()), cookie.clone())
-                .unwrap(),
+            eval_cookie_attribute_name(
+                CookieAttributeName::Path("_".to_string()),
+                cookie.clone(),
+                query_source_info
+            )
+            .unwrap()
+            .unwrap(),
             Value::String("/accounts".to_string())
         );
         assert_eq!(
             eval_cookie_attribute_name(
                 CookieAttributeName::MaxAge("_".to_string()),
                 cookie.clone(),
-            ),
+                query_source_info
+            )
+            .unwrap(),
             None
         );
         assert_eq!(
             eval_cookie_attribute_name(
                 CookieAttributeName::Expires("_".to_string()),
                 cookie.clone(),
+                query_source_info
             )
+            .unwrap()
             .unwrap(),
             Value::Date(
                 chrono::DateTime::parse_from_rfc2822("Wed, 13 Jan 2021 22:23:01 GMT")
@@ -975,7 +1033,9 @@ pub mod tests {
             eval_cookie_attribute_name(
                 CookieAttributeName::Secure("_".to_string()),
                 cookie.clone(),
+                query_source_info
             )
+            .unwrap()
             .unwrap(),
             Value::Unit
         );
@@ -983,13 +1043,47 @@ pub mod tests {
             eval_cookie_attribute_name(
                 CookieAttributeName::HttpOnly("_".to_string()),
                 cookie.clone(),
+                query_source_info
             )
+            .unwrap()
             .unwrap(),
             Value::Unit
         );
         assert_eq!(
-            eval_cookie_attribute_name(CookieAttributeName::SameSite("_".to_string()), cookie),
-            None
+            eval_cookie_attribute_name(
+                CookieAttributeName::SameSite("_".to_string()),
+                cookie,
+                query_source_info
+            )
+            .unwrap(),
+            None,
+        );
+    }
+
+    #[test]
+    fn test_eval_cookie_attribute_expires_error() {
+        let cookie = ResponseCookie {
+            name: "cookie1".to_string(),
+            value: "value1".to_string(),
+            attributes: vec![http::CookieAttribute {
+                name: "Expires".to_string(),
+                value: Some("???".to_string()),
+            }],
+        };
+        let query_source_info = SourceInfo::new(Pos::new(5, 1), Pos::new(5, 20));
+        let error = eval_cookie_attribute_name(
+            CookieAttributeName::Expires("_".to_string()),
+            cookie.clone(),
+            query_source_info,
+        )
+        .unwrap_err();
+        assert_eq!(
+            error.source_info,
+            SourceInfo::new(Pos::new(5, 1), Pos::new(5, 20))
+        );
+        assert_eq!(
+            error.kind,
+            RunnerErrorKind::Http(HttpError::CouldNotParseCookieExpires("???".to_string()))
         );
     }
 
@@ -1005,7 +1099,7 @@ pub mod tests {
                     value: QueryValue::Body,
                 },
                 &variables,
-                &http::hello_http_response(),
+                &[&http::hello_http_response()],
                 &mut cache,
             )
             .unwrap()
@@ -1018,7 +1112,7 @@ pub mod tests {
                 value: QueryValue::Body,
             },
             &variables,
-            &http::bytes_http_response(),
+            &[&http::bytes_http_response()],
             &mut cache,
         )
         .err()
@@ -1030,7 +1124,7 @@ pub mod tests {
         assert_eq!(
             error.kind,
             RunnerErrorKind::Http(HttpError::InvalidDecoding {
-                charset: "utf-8".to_string()
+                charset: "UTF-8".to_string()
             })
         );
     }
@@ -1040,18 +1134,18 @@ pub mod tests {
         let variables = VariableSet::new();
         let mut cache = BodyCache::new();
 
-        let http_response = http::Response {
+        let http_response = Response {
             body: vec![200],
             ..default_response()
         };
-        let error = eval_query(&xpath_users(), &variables, &http_response, &mut cache)
+        let error = eval_query(&xpath_users(), &variables, &[&http_response], &mut cache)
             .err()
             .unwrap();
         assert_eq!(error.source_info.start, Pos { line: 1, column: 1 });
         assert_eq!(
             error.kind,
             RunnerErrorKind::Http(HttpError::InvalidDecoding {
-                charset: "utf-8".to_string()
+                charset: "UTF-8".to_string()
             })
         );
     }
@@ -1082,11 +1176,11 @@ pub mod tests {
         let error = eval_query(
             &query,
             &variables,
-            &http::xml_two_users_http_response(),
+            &[&http::xml_two_users_http_response()],
             &mut cache,
         )
         .unwrap_err();
-        assert_eq!(error.kind, RunnerErrorKind::QueryInvalidXpathEval);
+        assert_eq!(error.kind, RunnerErrorKind::InvalidXPathEval);
         assert_eq!(error.source_info.start, Pos { line: 1, column: 7 });
     }
 
@@ -1099,7 +1193,7 @@ pub mod tests {
             eval_query(
                 &xpath_users(),
                 &variables,
-                &http::xml_two_users_http_response(),
+                &[&http::xml_two_users_http_response()],
                 &mut cache,
             )
             .unwrap()
@@ -1110,7 +1204,7 @@ pub mod tests {
             eval_query(
                 &xpath_count_user_query(),
                 &variables,
-                &http::xml_two_users_http_response(),
+                &[&http::xml_two_users_http_response()],
                 &mut cache,
             )
             .unwrap()
@@ -1151,7 +1245,7 @@ pub mod tests {
             eval_query(
                 &xpath_html_charset(),
                 &variables,
-                &http::html_http_response(),
+                &[&http::html_http_response()],
                 &mut cache,
             )
             .unwrap()
@@ -1187,7 +1281,7 @@ pub mod tests {
         let error = eval_query(
             &jsonpath_query,
             &variables,
-            &http::json_http_response(),
+            &[&http::json_http_response()],
             &mut cache,
         )
         .unwrap_err();
@@ -1210,13 +1304,18 @@ pub mod tests {
     fn test_query_invalid_json() {
         let variables = VariableSet::new();
         let mut cache = BodyCache::new();
-        let http_response = http::Response {
+        let http_response = Response {
             body: String::into_bytes(String::from("xxx")),
             ..default_response()
         };
-        let error = eval_query(&jsonpath_success(), &variables, &http_response, &mut cache)
-            .err()
-            .unwrap();
+        let error = eval_query(
+            &jsonpath_success(),
+            &variables,
+            &[&http_response],
+            &mut cache,
+        )
+        .err()
+        .unwrap();
         assert_eq!(error.source_info.start, Pos { line: 1, column: 1 });
         assert_eq!(error.kind, RunnerErrorKind::QueryInvalidJson);
     }
@@ -1226,12 +1325,18 @@ pub mod tests {
         let variables = VariableSet::new();
         let mut cache = BodyCache::new();
 
-        let http_response = http::Response {
+        let http_response = Response {
             body: String::into_bytes(String::from("{}")),
             ..default_response()
         };
         assert_eq!(
-            eval_query(&jsonpath_success(), &variables, &http_response, &mut cache).unwrap(),
+            eval_query(
+                &jsonpath_success(),
+                &variables,
+                &[&http_response],
+                &mut cache
+            )
+            .unwrap(),
             None
         );
     }
@@ -1245,7 +1350,7 @@ pub mod tests {
             eval_query(
                 &jsonpath_success(),
                 &variables,
-                &http::json_http_response(),
+                &[&http::json_http_response()],
                 &mut cache
             )
             .unwrap()
@@ -1256,7 +1361,7 @@ pub mod tests {
             eval_query(
                 &jsonpath_errors(),
                 &variables,
-                &http::json_http_response(),
+                &[&http::json_http_response()],
                 &mut cache
             )
             .unwrap()
@@ -1283,7 +1388,7 @@ pub mod tests {
             eval_query(
                 &regex_name(),
                 &variables,
-                &http::hello_http_response(),
+                &[&http::hello_http_response()],
                 &mut cache
             )
             .unwrap()
@@ -1294,7 +1399,7 @@ pub mod tests {
         let error = eval_query(
             &regex_invalid(),
             &variables,
-            &http::hello_http_response(),
+            &[&http::hello_http_response()],
             &mut cache,
         )
         .err()
@@ -1318,7 +1423,7 @@ pub mod tests {
                     value: QueryValue::Bytes,
                 },
                 &variables,
-                &http::hello_http_response(),
+                &[&http::hello_http_response()],
                 &mut cache,
             )
             .unwrap()
@@ -1346,10 +1451,10 @@ pub mod tests {
                     value: QueryValue::Sha256 {},
                 },
                 &variables,
-                &http::Response {
+                &[&Response {
                     body: vec![0xff],
                     ..default_response()
-                },
+                }],
                 &mut cache,
             )
             .unwrap()
@@ -1364,7 +1469,7 @@ pub mod tests {
     #[test]
     fn test_query_certificate() {
         assert!(eval_query_certificate(
-            &http::Response {
+            &Response {
                 ..default_response()
             },
             CertificateAttributeName::Subject
@@ -1373,7 +1478,7 @@ pub mod tests {
         .is_none());
         assert_eq!(
             eval_query_certificate(
-                &http::Response {
+                &Response {
                     certificate: Some(http::Certificate {
                         subject: "A=B, C=D".to_string(),
                         issuer: String::new(),
